<?php

namespace IQBall\App;

use IQBall\Core\Action;
use IQBall\Core\Http\HttpCodes;
use IQBall\Core\Http\HttpResponse;
use IQBall\Core\Http\JsonHttpResponse;
use IQBall\Core\Session\MutableSessionHandle;
use IQBall\Core\Validation\ValidationFail;
use Twig\Environment;
use Twig\Error\LoaderError;
use Twig\Error\RuntimeError;
use Twig\Error\SyntaxError;
use Twig\Loader\FilesystemLoader;

class App {
    public static function render(HttpResponse $response): void {
        http_response_code($response->getCode());

        foreach ($response->getHeaders() as $header => $value) {
            header("$header: $value");
        }

        if ($response instanceof ViewHttpResponse) {
            self::renderView($response);
        } elseif ($response instanceof JsonHttpResponse) {
            header('Content-type: application/json');
            echo $response->getJson();
        }
    }

    private static function renderView(ViewHttpResponse $response): void {
        $file = $response->getFile();
        $args = $response->getArguments();

        switch ($response->getViewKind()) {
            case ViewHttpResponse::REACT_VIEW:
                send_react_front($file, $args);
                break;
            case ViewHttpResponse::TWIG_VIEW:
                try {
                    $loader = new FilesystemLoader('../src/App/Views/');
                    $twig = new Environment($loader);
                    $twig->display($file, $args);
                } catch (RuntimeError | SyntaxError | LoaderError $e) {
                    http_response_code(500);
                    echo "There was an error rendering your view, please refer to an administrator.\nlogs date: " . date("YYYD, d M Y H:i:s");
                    throw $e;
                }
                break;
        }
    }

    /**
     * @param Action<MutableSessionHandle> $action
     * @param mixed[] $params
     * @param MutableSessionHandle $session
     * @return HttpResponse
     */
    private static function runAction(Action $action, array $params, MutableSessionHandle $session): HttpResponse {
        global $basePath;
        if ($action->isAuthRequired()) {
            $account = $session->getAccount();
            if ($account == null) {
                // put in the session the initial url the user wanted to get
                $session->setInitialTarget($_SERVER['REQUEST_URI']);
                return HttpResponse::redirect($basePath . "/login");
            }
        }

        return $action->run($params, $session);
    }

    /**
     * @param array<string, mixed>|false $match
     * @param MutableSessionHandle $session
     * @return HttpResponse
     */
    public static function runMatch($match, MutableSessionHandle $session): HttpResponse {
        if (!$match) {
            return ViewHttpResponse::twig("error.html.twig", [
                'failures' => [ValidationFail::notFound("Could not find page ${_SERVER['REQUEST_URI']}.")],
            ], HttpCodes::NOT_FOUND);
        }

        return self::runAction($match['target'], $match['params'], $session);
    }

}
